'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Parser = function () {
  function Parser() {
    var config = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    _classCallCheck(this, Parser);

    var defaultConfig = {
      objectDelimiters: ['{0}', '{1}', '{2}'],
      arrayDelimiters: ['[0]', '[1]', '[2]']
    };
    this.config = Object.assign({}, defaultConfig, config);
    if (!config.schema) {
      throw 'Schema is required';
    }
  }

  _createClass(Parser, [{
    key: '_getDelimiter',
    value: function _getDelimiter(type, level) {
      if (type === 'array' && this.config.arrayDelimiters[level]) {
        return this.config.arrayDelimiters[level];
      }
      if (type === 'object' && this.config.objectDelimiters[level]) {
        return this.config.objectDelimiters[level];
      }
      throw 'No ' + type + ' delimiter specified for level ' + level;
    }
  }, {
    key: '_stringifyArray',
    value: function _stringifyArray() {
      var array = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : [];

      var _this = this;

      var items = arguments[1];
      var level = arguments[2];

      var delimiter = this._getDelimiter('array', level);
      var output = void 0;
      var dataType = items.type;
      switch (dataType) {
        case 'object':
          output = array.map(function (item) {
            return _this._stringifyObject(item, items.properties, level + 1);
          });
          break;
        case 'array':
          output = array.map(function (item) {
            return _this._stringifyArray(item, items.items, level + 1);
          });
          break;
        default:
          output = array;
      }
      return output.join(delimiter);
    }
  }, {
    key: '_parseArray',
    value: function _parseArray() {
      var string = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      var _this2 = this;

      var items = arguments[1];
      var level = arguments[2];

      var delimiter = this._getDelimiter('array', level);
      var array = string.split(delimiter);
      var dataType = items.type;
      var output = void 0;
      switch (dataType) {
        case 'object':
          output = array.map(function (item) {
            return _this2._parseObject(item, items.properties, level + 1);
          });
          break;
        case 'array':
          output = array.map(function (item) {
            return _this2._parseArray(item, items.items, level + 1);
          });
          break;
        default:
          output = array;
      }
      return output;
    }
  }, {
    key: '_stringifyObject',
    value: function _stringifyObject() {
      var object = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

      var _this3 = this;

      var properties = arguments[1];
      var level = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      var delimiter = this._getDelimiter('object', level);
      var output = Object.keys(properties).map(function (key) {
        var dataType = properties[key].type;
        switch (dataType) {
          case 'object':
            return _this3._stringifyObject(object[key], properties[key].properties, level + 1);
          case 'array':
            return _this3._stringifyArray(object[key], properties[key].items, level);
          default:
            return object[key];
        }
      });
      return output.join(delimiter);
    }
  }, {
    key: '_parseObject',
    value: function _parseObject() {
      var string = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : '';

      var _this4 = this;

      var properties = arguments[1];
      var level = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 0;

      var delimiter = this._getDelimiter('object', level);
      var objectArray = string.split(delimiter);
      var result = {};
      Object.keys(properties).forEach(function (key, i) {
        var dataType = properties[key].type;
        switch (dataType) {
          case 'object':
            result[key] = _this4._parseObject(objectArray[i], properties[key].properties, level + 1);
            break;
          case 'array':
            result[key] = _this4._parseArray(objectArray[i], properties[key].items, level);
            break;
          case 'integer':
            result[key] = parseInt(objectArray[i]);
            break;
          default:
            result[key] = objectArray[i];
        }
      });
      return result;
    }
  }, {
    key: 'stringify',
    value: function stringify(data) {
      return this._stringifyObject(data, this.config.schema);
    }
  }, {
    key: 'parse',
    value: function parse(string) {
      return this._parseObject(string, this.config.schema);
    }
  }]);

  return Parser;
}();

module.exports = Parser;